@@ -23,6 +23,7 @@
 #include <string>
 #include <iostream>
 #include <map>
+#include <vector>
 //#include <ext/hash_map>
 
 using namespace std;
@@ -60,7 +61,6 @@ class lf /*: public lockable*/ {
    public:
       lf(const char *path) { 
          csize=ncsize; // default for writters, can be changed by existing value from the first chunk
-         ccnt=0;
          cache_fd=-1;
          cache_pos=-1;
 
@@ -70,7 +70,7 @@ class lf /*: public lockable*/ {
          virtname=fromdir+path;
 
          if(0==lstat(virtname.c_str(), &stdata)) { // unchunked file, nothing to do here
-            ccnt=1;
+            fflags.resize(1);
             return;
          }
 
@@ -82,17 +82,18 @@ class lf /*: public lockable*/ {
          // .aa defines the chunk size and all usefull attributes
 
 
-         csize=stdata.st_size; // chunk size to be saved as default for this object
+         if(stdata.st_size)
+            csize=stdata.st_size; // chunk size to be saved as default for this object
          stdata.st_size=0; // not double-counting the first chunk
          for(unsigned int i=0;i<=676;i++) {
             if(!lstat(makeChunkName(i).c_str(), &stbuf)) {
                if( (uint64_t)stbuf.st_size > csize) {
                   // FIXME, report error and mark invalid
-                  ccnt=0;
+                  fflags.clear();
                }
                else if( (uint64_t)stbuf.st_size < csize) {
                   // last chunk
-                  ccnt=i+1;
+                  fflags.resize(i+1);
                   stdata.st_size+=stbuf.st_size;
                   return;
                }
@@ -100,7 +101,7 @@ class lf /*: public lockable*/ {
                   stdata.st_size+=stbuf.st_size;
             }
             else {
-               ccnt=i;
+               fflags.resize(i);
                return;
             }
          }
@@ -108,11 +109,11 @@ class lf /*: public lockable*/ {
 
       inline string getCname(unsigned int i) {
          //setLockGuard;
-         if(ccnt<2)
-            return virtname;
-         if(i>=ccnt)
+         if(i>=fflags.size())
             return "";
-         return  makeChunkName(i);
+         if(fflags.size()<2)
+            return virtname;
+         return makeChunkName(i);
       }
 
       ~lf() { 
@@ -125,13 +126,13 @@ class lf /*: public lockable*/ {
       }
 
       bool exists() { 
-         return (ccnt>0); 
+         return (fflags.size()); 
       }
 
       bool prep_open(int flags) {
-         if(ccnt<1) {
+         if(fflags.empty()) {
             if((flags&O_CREAT))
-               ccnt=1; // force the use the virtname for first chunk
+               fflags.resize(1); // force the use the virtname for first chunk
             else {
                errno=ENOENT;
                return false;
@@ -177,13 +178,18 @@ class lf /*: public lockable*/ {
          pthread_mutex_lock(&chlock);
          // pad preceeding, truncate the last one, drop the remaining rest
          unsigned int i;
-         for(i=0;i<chunks;i++)
-            truncate(getCname(i).c_str(), csize);
+         for(i=0;i<chunks;i++) {
+            if(!fflags[i]) {
+               truncate(getCname(i).c_str(), csize);
+               fflags[i]=true;
+            }
+         }
          //cerr << "truncating " << getCname(i).c_str() << " to " <<coff <<endl;
          truncate(getCname(i).c_str(), coff);
-         for(i++;i<ccnt;i++)
+         fflags[i]=false; // don't know for sure, make it so
+         for(i++;i<fflags.size();i++)
             unlink(getCname(i).c_str());
-         ccnt=chunks+1;
+         fflags.resize(chunks+1); // update count
          stdata.st_size=size;
          // cerr << "neu: " << ccnt << " chunks, " << size << " bytes\n";
          pthread_mutex_unlock(&chlock);
@@ -199,18 +205,22 @@ class lf /*: public lockable*/ {
                close(cache_fd);
             // getCname resolution must be valid when open happens
             pthread_mutex_lock(&chlock);
-            if(coff==0 && chunk == 1 && ccnt==1) {
+            if(coff==0 && chunk == 1 && fflags.size()==1) {
                 // time to rename, must be locked here
                 rename(virtname.c_str(), (virtname+".aa").c_str()); // FIXME: error handling
             }
-            if(chunk>=ccnt)
-               ccnt=chunk+1;
+            if(chunk>=fflags.size())
+               fflags.resize(chunk+1);
 
-            // make sure the previous have the right size. This is cheap and
-            // dirty and not wise aka sparse-file friendly.
-            if(chunk>0)
-               for(unsigned int i=0;i<chunk;i++)
-                  truncate(getCname(i).c_str(), csize);
+            // make sure the previous have the right size.
+            if(chunk>0) {
+               for(unsigned int i=0;i<chunk;i++) {
+                  if(!fflags[i]) {
+                     truncate(getCname(i).c_str(), csize);
+                     fflags[i]=true;
+                  }
+               }
+            }
             
             //cerr << "> " << getCname(chunk).c_str() << " flags: " << (flags|O_CREAT) <<endl;
             cache_fd = open(getCname(chunk).c_str(), O_CREAT|O_RDWR, S_IRUSR|S_IWUSR|S_IRGRP);
@@ -236,7 +246,7 @@ class lf /*: public lockable*/ {
       string virtname;
       uint64_t csize;
       struct stat stdata;
-      unsigned int ccnt;
+      vector<bool> fflags; // flag bits for "full files" to speedup padding (which is O(n^2) * syscall time) ; it also serves as chunk counter
 
       inline string makeChunkName(unsigned int i) {
          string ret=virtname;
@@ -552,13 +562,12 @@ static int fulfs_ftruncate(const char *path, off_t size,
 static int fulfs_utime(const char *path, struct utimbuf *buf)
 {
    lf p(path);
-   string n=p.getCname(0);
-   int res = utime((fromdir+path).c_str(), buf);
-   if(res<0)
+   if( utime(p.getCname(0).c_str(), buf) < 0)
       return -errno;
    // rest does not matter
    for(int i=1;;i++) {
       string n=p.getCname(i);
+      cerr << n<<endl;
       if(n.empty())
          break;
       utime(n.c_str(), buf);
@@ -746,23 +755,23 @@ int main(int argc, char *argv[])
    umask(0);
 
    if(argc<3)
-      barf("Needs a source and a target directory, see --help.");
+      barf("\nNeeds a source and a target directory, see --help.\n");
 
    struct stat stbuf;
    if(stat(argv[1], &stbuf) || !S_ISDIR(stbuf.st_mode))
-      barf(argv[1] << " is not a directory.");
+      barf("\n" << argv[1] << " is not a directory.\n\n");
 
    char *buf=new char[PATH_MAX+1];
    if(realpath(argv[1], buf))
       fromdir=buf;
    else {
-      cerr << "Invalid source directory, " << argv[1] <<endl;
+      cerr << "\nInvalid source directory, " << argv[1] <<endl <<endl;
       exit(EXIT_FAILURE);
    }
    delete [] buf;
 
    if(stat(argv[2], &stbuf) || !S_ISDIR(stbuf.st_mode))
-      barf(argv[2] << " is not a directory.");
+      barf(endl<< argv[2] << " is not a directory.\n\n");
 
    todir=argv[2];
 
@@ -805,8 +814,8 @@ int main(int argc, char *argv[])
             default:
                barf(p << " is not a valid unit.");
          }
+         ncsize=testsize;
       }
-      ncsize=testsize;
    }
 
    /* hide the from argument */
